"
COPYRIGHT (c) 2020 LabWare
"
"{ Package: 'stx:goodies/ring/core' }"

"{ NameSpace: Smalltalk }"

Announcer subclass:#RGEnvironmentAnnouncer
	instanceVariableNames:'suspended private storedAnnouncements'
	classVariableNames:''
	poolDictionaries:''
	category:'Ring-Core-Announcements'
!

RGEnvironmentAnnouncer class instanceVariableNames:'announcer'

"
 No other class instance variables are inherited by this class.
"
!

RGEnvironmentAnnouncer comment:'This class plays the role of the announcer for events raised by the Pharo system.

BEWARE: You should not need to subclass me. Think about just using me to send your announces (if you need system behavior) or using your own announcers as instance or class  variables.'
!

!RGEnvironmentAnnouncer class methodsFor:'documentation'!

copyright
"
COPYRIGHT (c) 2020 LabWare


"
! !

!RGEnvironmentAnnouncer methodsFor:'accessing'!

private
    ^private ifNil: [ private := Announcer new ]
! !

!RGEnvironmentAnnouncer methodsFor:'announce'!

announce: anAnnouncement
    self isSuspended 
        ifFalse: [ 
            self private announce: anAnnouncement.
            super announce: anAnnouncement ]
        ifTrue:[
            storedAnnouncements ifNotNil:[ storedAnnouncements add: anAnnouncement ]
        ].
!

suspendAllWhile: aBlock
    | oldSuspended |
    oldSuspended := self isSuspended.
    suspended := true.
    ^aBlock ensure: [ suspended := oldSuspended ]
!

suspendAllWhileStoring: aBlock
    | reentring |
    " Suspend all the announcements, storing them in an OrderedCollection, then returns this collection"
    
    reentring := storedAnnouncements isNotNil.
    
    reentring ifFalse:[
        storedAnnouncements := OrderedCollection new.
    ].

    [ 
        self suspendAllWhile: aBlock.
        ^ storedAnnouncements.
    ] ensure:[
        reentring ifFalse:[
            storedAnnouncements := nil.
        ]
    ]
! !

!RGEnvironmentAnnouncer methodsFor:'error handling'!

behaviorAdded: anRGBehavior

"/    self announce: (ClassAdded class: anRGBehavior category: nil)

    "Modified: / 22-12-2020 / 11:01:36 / Jan Vrany <jan.vrany@labware.com>"
!

behaviorCommentModified: anRGBehavior

"/    self announce: (ClassCommented classCommented: anRGBehavior)

    "Modified: / 22-12-2020 / 11:01:41 / Jan Vrany <jan.vrany@labware.com>"
!

behaviorDefinitionChangedFrom: oldRGBehavior to: newRGBehavior

"/    self announce: (ClassModifiedClassDefinition  
"/        classDefinitionChangedFrom:  oldRGBehavior
"/        to: newRGBehavior)

    "Modified: / 22-12-2020 / 11:01:48 / Jan Vrany <jan.vrany@labware.com>"
!

behaviorDefinitionModified: anRGBehavior

"/    self announce: (ClassModificationApplied toClass: anRGBehavior)

    "Modified: / 22-12-2020 / 11:02:09 / Jan Vrany <jan.vrany@labware.com>"
!

behaviorModificationAppliedTo: anRGBehavior

"/    self announce: (ClassModificationApplied toClass: anRGBehavior)

    "Modified: / 22-12-2020 / 11:02:15 / Jan Vrany <jan.vrany@labware.com>"
!

behaviorParentRenamed: anRGBehavior from: oldName

"/    self announce: (ClassParentRenamed 
"/        classParentOf: anRGBehavior
"/        renamedFrom: oldName
"/        to: anRGBehavior name)

    "Modified: / 22-12-2020 / 11:02:19 / Jan Vrany <jan.vrany@labware.com>"
!

behaviorRecategorized: anRGBehavior 

"/    self announce: (ClassRecategorized 
"/        class: anRGBehavior 
"/        recategorizedFrom: nil 
"/        to: anRGBehavior category)

    "Modified: / 22-12-2020 / 11:02:25 / Jan Vrany <jan.vrany@labware.com>"
!

behaviorRemoved: anRGBehavior

"/    self announce: (ClassRemoved 
"/        class: anRGBehavior category: anRGBehavior category)

    "Modified: / 22-12-2020 / 11:02:30 / Jan Vrany <jan.vrany@labware.com>"
!

behaviorRenamed: anRGBehavior from: oldName

"/    self announce: (ClassRenamed 
"/        class: anRGBehavior 
"/        category: anRGBehavior category 
"/        oldName: oldName 
"/        newName: anRGBehavior name)

    "Modified: / 22-12-2020 / 11:02:35 / Jan Vrany <jan.vrany@labware.com>"
!

behaviorReorganized: anRGBehavior

"/    self announce: (ClassReorganized class: anRGBehavior)

    "Modified: / 22-12-2020 / 11:02:39 / Jan Vrany <jan.vrany@labware.com>"
!

behaviorRepackaged: anRGBehavior from: oldPackage

"/    self announce: (ClassRepackaged 
"/        classRepackaged: anRGBehavior
"/        oldPackage: oldPackage
"/        newPackage: anRGBehavior package)

    "Created: / 22-12-2020 / 10:55:07 / Jan Vrany <jan.vrany@labware.com>"
!

methodAdded: aMethod 

"/    self announce: (MethodAdded method: aMethod)

    "Modified: / 22-12-2020 / 11:02:42 / Jan Vrany <jan.vrany@labware.com>"
!

methodRecategorized: anRGMethod from: oldProtocol

"/    self announce: (MethodRecategorized method: self oldProtocol: oldProtocol)

    "Created: / 22-12-2020 / 10:57:19 / Jan Vrany <jan.vrany@labware.com>"
!

methodRemoved: aMethod 

"/    self announce: (MethodRemoved methodRemoved: aMethod protocol: aMethod protocol origin: aMethod parent)

    "Modified: / 22-12-2020 / 11:02:46 / Jan Vrany <jan.vrany@labware.com>"
!

packageAdded: anRGPackage

"/    self announce: (RPackageRegistered to: anRGPackage)

    "Created: / 22-12-2020 / 10:59:38 / Jan Vrany <jan.vrany@labware.com>"
!

packageRemoved: anRGPackage

"/    self announce: (RPackageUnregistered to: anRGPackage)

    "Created: / 22-12-2020 / 10:58:41 / Jan Vrany <jan.vrany@labware.com>"
!

packageRenamed: anRGPackage from: oldName

"/    self announce: (RPackageRenamed 
"/            to: anRGPackage
"/            oldName: oldName 
"/            newName: anRGPackage name)

    "Created: / 22-12-2020 / 10:50:30 / Jan Vrany <jan.vrany@labware.com>"
! !

!RGEnvironmentAnnouncer methodsFor:'subscription'!

unsubscribe: anObject
    self private unsubscribe: anObject.	
    super unsubscribe: anObject.
! !

!RGEnvironmentAnnouncer methodsFor:'testing'!

isSuspended
    ^suspended ifNil: [ suspended := false ]
! !

!RGEnvironmentAnnouncer class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

