"
COPYRIGHT (c) 2020 LabWare
"
"{ Package: 'stx:goodies/ring/core' }"

"{ NameSpace: Smalltalk }"

RGReadOnlyBackend subclass:#RGReadOnlyImageBackend
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Ring-Core-Environment'
!

RGReadOnlyImageBackend comment:''
!

!RGReadOnlyImageBackend class methodsFor:'documentation'!

copyright
"
COPYRIGHT (c) 2020 LabWare


"
! !

!RGReadOnlyImageBackend class methodsFor:'as yet unclassified'!

for: anRGEnvironment

    ^ self new 
        environment: anRGEnvironment;
        yourself.
! !

!RGReadOnlyImageBackend methodsFor:'as yet unclassified'!

definitionFor: anObject 

    ^ self definitionFor: anObject ifAbsentRegister: [ anObject ensureRingDefinitionIn: self environment ]
!

realBehaviorFor: anRGBehavior

    "TODO: should we cache the definition?"
    ^ Smalltalk classOrTraitNamed: anRGBehavior name asSymbol
!

realMethodFor: anRGMethod

    "TODO: should we cache the definition?"
    ^ ((self realBehaviorFor: anRGMethod parent) >> anRGMethod selector)
    
! !

!RGReadOnlyImageBackend methodsFor:'behavior'!

layoutFor: anRGBehavior

    | realClass realLayout def |
    
    realClass := self realBehaviorFor: anRGBehavior.
    realLayout := realClass classLayout.
    def := self 	definitionFor: realLayout ifAbsentRegister: [ 
            realLayout asRingMinimalDefinitionIn: anRGBehavior environment ].
    ^ def
!

localMethodsFor: anRGBehavior do: aBlock

    (self realBehaviorFor: anRGBehavior) localMethods do: [:method |
        | def |
        def := self definitionFor: method ifAbsentRegister: [
            method asRingMinimalDefinitionIn: anRGBehavior environment].
        aBlock value: def]
!

metaclassFor: anRGBehavior

    | realBehavior realMetaclass def |
    
    realBehavior := self realBehaviorFor: anRGBehavior.
    realMetaclass := realBehavior class.
    def := self 	definitionFor: realMetaclass ifAbsentRegister: [ 
            realMetaclass asRingMinimalDefinitionIn: anRGBehavior environment ].
    ^ def
!

superclassFor: anRGBehavior

    | realClass realSuperclass def |
    
    realClass := self realBehaviorFor: anRGBehavior.
    realSuperclass := realClass superclass.
    realSuperclass ifNil: [ realSuperclass := realClass ].
    def := self 	definitionFor: realSuperclass ifAbsentRegister: [ 
            realSuperclass asRingMinimalDefinitionIn: anRGBehavior environment ].
    ^ def
!

tagsForClass: anRGBehavior do: aBlock

    | realClass |
    
    realClass := self realBehaviorFor: anRGBehavior.
    realClass tags do: [:tag | aBlock value: tag ]
!

traitCompositionFor: anRGBehavior

    | realClass realTraitComposition def |
    
    realClass := self realBehaviorFor: anRGBehavior.
    realTraitComposition := realClass traitComposition.
    def := self 	definitionFor: realTraitComposition ifAbsentRegister: [ 
            realTraitComposition asRingMinimalDefinitionIn: anRGBehavior environment ].
    ^ def
! !

!RGReadOnlyImageBackend methodsFor:'class'!

categoryFor: anRGBehavior

    ^ (self realBehaviorFor: anRGBehavior) category
!

classVariablesFor: anRGBehavior do: aBlock

    | realClass |
    realClass := self realBehaviorFor: anRGBehavior.
    realClass classVariables do: [ :classVariable | 
        | def |
        def := RGClassVariable named: classVariable key parent: anRGBehavior.
        aBlock value: def. ]
!

commentFor: anRGBehavior

    ^  (self realBehaviorFor: anRGBehavior) asRingMinimalCommentDefinitionIn: self environment
!

packageFor: anRGBehavior

    ^ (self realBehaviorFor: anRGBehavior) package asRingMinimalDefinitionIn: anRGBehavior environment.
!

sharedPoolsFor: anRGBehavior do: aBlock

    ^ anRGBehavior behaviorStrategy pvtSharedPoolsDo: aBlock
! !

!RGReadOnlyImageBackend methodsFor:'class comment'!

classCommentAuthorFor: anRGComment

    ^ RGStampParser authorForStamp: (self realBehaviorFor: anRGComment parent) organization commentStamp.
!

classCommentContentFor: anRGComment

    ^ (self realBehaviorFor: anRGComment parent) organization classComment.
!

classCommentTimeFor: anRGComment

    ^ RGStampParser timeForStamp: (self realBehaviorFor: anRGComment parent) organization commentStamp.
! !

!RGReadOnlyImageBackend methodsFor:'class description'!

metaClassFor: anRGBehavior

    | realClass realMetaclass def |
    
    realClass := self realBehaviorFor: anRGBehavior.
    realMetaclass := realClass class.
    def := self 	definitionFor: realMetaclass ifAbsentRegister: [ 
            realMetaclass asRingMinimalDefinitionIn: anRGBehavior environment ].
    ^ def
! !

!RGReadOnlyImageBackend methodsFor:'environment'!

behaviorsFor: anRGEnvironment do: aBlock

    SystemNavigation default allBehaviors do: [:each |
        | def | 
        def := self definitionFor: each ifAbsentRegister: [each asRingMinimalDefinitionIn: anRGEnvironment].
        aBlock value: def.].
!

globalVariablesFor: anRGEnvironment do: aBlock

    Smalltalk globals associations do: [:each |
        | def | 
        def := self definitionFor: each ifAbsentRegister: [each asRingMinimalDefinitionIn: anRGEnvironment].
        aBlock value: def.].
! !

!RGReadOnlyImageBackend methodsFor:'error handling'!

packagesFor: anRGEnvironment do: aBlock

    RPackageOrganizer default packagesDo: [:each |
        | def | 
        def := self definitionFor: each ifAbsentRegister: [each asRingMinimalDefinitionIn: anRGEnvironment].
        aBlock value: def.].
!

realPackageFor: anRGPackage

    ^ RPackageOrganizer default packageNamed: anRGPackage name ifAbsent: [nil].
! !

!RGReadOnlyImageBackend methodsFor:'general'!

nameFor: anRGObject

    ^ anRGObject pvtName
! !

!RGReadOnlyImageBackend methodsFor:'initialization'!

initialize

    super initialize.
    
    providedDefinitions := IdentityDictionary new.
    
! !

!RGReadOnlyImageBackend methodsFor:'metaclass'!

baseClassFor: anRGMetaclassTrait

    ^ (self realBehaviorFor: anRGMetaclassTrait) baseClass asRingMinimalDefinitionIn: anRGMetaclassTrait environment.
! !

!RGReadOnlyImageBackend methodsFor:'metaclass trait'!

metaclassTraitBaseTraitFor: anRGMetaclassTrait
    ^ (self realBehaviorFor: anRGMetaclassTrait) instanceSide
        asRingMinimalDefinitionIn: anRGMetaclassTrait environment
! !

!RGReadOnlyImageBackend methodsFor:'method'!

astFor: anRGMethod

    ^  anRGMethod astFromSource
!

authorFor: anRGMethod

    ^ RGStampParser authorForStamp: (self realMethodFor: anRGMethod) stamp
!

hasSourceCodeFor: anRGMethod 

    ^ (self realMethodFor: anRGMethod) sourceCode
!

methodPackageFor: anRGMethod

    ^ (self realMethodFor: anRGMethod) package asRingMinimalDefinitionIn: anRGMethod environment.
!

sourceCodeFor: anRGMethod 

    ^ (self realMethodFor: anRGMethod) sourceCode
!

tagsForMethod: anRGMethod do: aBlock

    | realMethod |
    
    realMethod := self realMethodFor: anRGMethod.
    { realMethod protocol } do: aBlock.
!

tagsForMethodsFor: anRGBehavior do: aBlock

    (self realBehaviorFor: anRGBehavior) protocols do: aBlock
!

timeFor: anRGMethod

    ^ RGStampParser timeForStamp: (self realMethodFor: anRGMethod) stamp
! !

!RGReadOnlyImageBackend methodsFor:'package'!

definedBehaviorsFor: anRGPackage do: aBlock

    | realPackage |
    
    realPackage := self realPackageFor: anRGPackage.
    realPackage ifNotNil: [ 
        realPackage definedClassesDo: [:behaviorName |
            | def cls | 
            cls := Smalltalk classOrTraitNamed: behaviorName.
            def := self definitionFor: cls ifAbsentRegister: [cls asRingMinimalDefinitionIn: anRGPackage environment].
            aBlock value: def.]]
!

extensionMethodsFor: anRGPackage do: aBlock

    | realPackage |
    
    realPackage := self realPackageFor: anRGPackage.
    realPackage ifNotNil: [ 
        realPackage extensionMethods do: [:aCompiledMethod |
            | def | 
            def := self definitionFor: aCompiledMethod ifAbsentRegister: [aCompiledMethod asRingMinimalDefinitionIn: anRGPackage environment].
            aBlock value: def.]]
! !

!RGReadOnlyImageBackend methodsFor:'pointer layout'!

slotsFor: anRGLayout do: aBlock
    | realClass |
    realClass := self realBehaviorFor: anRGLayout parent.
    realClass classLayout slots
        do: [ :each | 
            | def |
            def := self
                definitionFor: each
                ifAbsentRegister: [ each asRingMinimalDefinitionIn: anRGLayout environment ].
            aBlock value: def ]
! !

!RGReadOnlyImageBackend methodsFor:'resolving'!

hasFullyResolved: anRGObject

    ^ true
!

hasFullyUnresolved: anRGObject		
        
    ^ false
!

hasResolved: anRGObject

    ^ true
!

resolvedPropertiesFor: anRGObject

    ^ anRGObject pvtResolvableProperties collect: [:each | each key]
!

unresolvedPropertiesFor: anRGObject

    ^ Dictionary new.
! !

!RGReadOnlyImageBackend methodsFor:'slot'!

expressionFor: anRGUnknownSlot

    | realClass realSlot |
    
    realClass := self realBehaviorFor: anRGUnknownSlot parent parent.
    realSlot := realClass slotNamed: anRGUnknownSlot name.
    ^ (realSlot printString copyAfter: $>) trimBoth
! !

!RGReadOnlyImageBackend methodsFor:'trait'!

classTraitFor: anRGTrait

    ^ (self realBehaviorFor: anRGTrait) classTrait asRingMinimalDefinitionIn: anRGTrait environment.
!

traitCommentFor: anRGBehavior

    ^  (self realBehaviorFor: anRGBehavior) asRingMinimalCommentDefinitionIn: self environment
!

traitPackageFor: anRGBehavior

    ^ (self realBehaviorFor: anRGBehavior) package asRingMinimalDefinitionIn: anRGBehavior environment.
! !

!RGReadOnlyImageBackend methodsFor:'trait alias'!

aliasesFor: anRGTraitAlias do: aBlock

    ^ (anRGTraitAlias propertyNamed: #realObject ifAbsent: [ self error: 'You can use only trait transformations generated directly by this backend' ]) aliases associations do: [:each | aBlock value: each ]
! !

!RGReadOnlyImageBackend methodsFor:'trait composition'!

transformationsFor: anRGTraitComposition do: aBlock

    | realBehavior realTraitComposition |
    "^ anRGTraitComposition pvtTransformationsDo: aBlock"

    realBehavior := self realBehaviorFor: anRGTraitComposition parent.
    realTraitComposition := realBehavior traitComposition.

    realTraitComposition transformations do: [:transformation |
        | def |
        def := self definitionFor: transformation ifAbsentRegister: [
            transformation asRingMinimalDefinitionIn: anRGTraitComposition environment].
        aBlock value: def]
! !

!RGReadOnlyImageBackend methodsFor:'trait exclusion'!

exclusionsFor: anRGTraitExclusion do: aBlock

    ^ (anRGTraitExclusion propertyNamed: #realObject ifAbsent: [ self error: 'You can use only trait transformations generated directly by this backend' ]) exclusions do: [:each | aBlock value: each ]
! !

!RGReadOnlyImageBackend methodsFor:'trait transormation'!

subjectFor: anRGTraitTransformation

    ^ (anRGTraitTransformation propertyNamed: #realObject ifAbsent: [ self error: 'You can use only trait transformations generated directly by this backend' ]) subject asRingMinimalDefinitionIn: anRGTraitTransformation environment
! !

!RGReadOnlyImageBackend methodsFor:'unresolved objects'!

createUnresolvedClassGroupFor: anRGBehavior 

    ^ 	anRGBehavior isMeta
        ifFalse: [ anRGBehavior ]
        ifTrue: [ anRGBehavior metaclass].
! !

