"
COPYRIGHT (c) 2020 LabWare
"
"{ Package: 'stx:goodies/ring/core' }"

"{ NameSpace: Smalltalk }"

RGObject subclass:#RGTraitComposition
	instanceVariableNames:'transformations'
	classVariableNames:''
	poolDictionaries:''
	category:'Ring-Core-Kernel'
!

RGTraitComposition comment:''
!

!RGTraitComposition class methodsFor:'documentation'!

copyright
"
COPYRIGHT (c) 2020 LabWare

"
! !

!RGTraitComposition methodsFor:'accessing - backend'!

addTransformation: anRGTraitTransformation 

    self parent announceDefinitionChangeDuring: [ 
        self backend forBehavior addTransformation: anRGTraitTransformation to: self ].
!

cleanTransformations

    self parent announceDefinitionChangeDuring: [ 
        self backend forBehavior cleanTransformationsFor: self ].
!

removeTransformation: anRGTraitTransformation

    self parent announceDefinitionChangeDuring: [ 
        self backend forBehavior removeTransformation: anRGTraitTransformation from: self ].
!

transformationsDo: aBlock

    self backend forBehavior transformationsFor: self do: aBlock
! !

!RGTraitComposition methodsFor:'as yet unclassified'!

defaultTransformations

    ^ OrderedCollection new.
!

referencedBehaviors

    ^ self transformations collect: [ :each |
        each isTrait 
            ifTrue: [ each ]
            ifFalse: [ each subject ] ]
!

traitCompositionString

    self transformations isEmpty ifTrue: [ ^ '{}' ].
    
    ^String streamContents: [ :stream | 
        self transformations 
            do: [ :each | stream nextPutAll: each traitTransformationString ]
            separatedBy: [ stream nextPutAll: ' + ' ] ]	
    
!

traits

    ^ self transformations collect: [:each | each trait]
!

usedTraits

    ^ self transformations flatCollect: [ :each | 
        each isTrait 
            ifTrue: [ {each} ]
            ifFalse: [ each usedTraits ].
         ] as: IdentitySet.
! !

!RGTraitComposition methodsFor:'initialization'!

initialize

    super initialize.
    
    transformations := self unresolvedValue: self defaultTransformations.
!

initializeUnresolved

    super initializeUnresolved.
    
    transformations := self unresolvedValue: self defaultTransformations.
! !

!RGTraitComposition methodsFor:'managing container'!

addoptToParentStub

    super addoptToParentStub.

    self environment backend createUnresolvedClassGroupFor: self parent
!

defaultParentStub

    ^ self defaultClassStub
! !

!RGTraitComposition methodsFor:'private - backend access'!

pvtAddTransformation: anRGTraitTransformation

    self environment verifyOwnership: anRGTraitTransformation.
    
    transformations isRingResolved ifFalse: [ 
        self pvtCleanTransformations  ].
    
    transformations add: anRGTraitTransformation.	
!

pvtCleanTransformations

    transformations := self defaultTransformations.
!

pvtRemoveTransformation: anRGTraitTransformation

    self environment verifyOwnership: anRGTraitTransformation.
    
    transformations remove: anRGTraitTransformation.
!

pvtResolvableProperties

    ^ super pvtResolvableProperties, { 
        #transformations -> transformations. 
    }
    
    
!

pvtTransformationsDo: aBlock

    transformations value do: aBlock
! !

!RGTraitComposition methodsFor:'queries - methods'!

methods

    ^ Array streamContents: [ :stream | 
        self transformationsDo: [ :transformation | stream nextPutAll: transformation methods ]]
! !

!RGTraitComposition methodsFor:'queries - transformations'!

transformations

    | allTransformations |
    allTransformations := OrderedCollection new.
    self transformationsDo: [ :each | allTransformations add: each].
    ^ allTransformations
! !

!RGTraitComposition methodsFor:'resolving'!

makeResolved

    super makeResolved.
    
    transformations := self transformations markAsRingResolved.
! !

!RGTraitComposition methodsFor:'testing types'!

isTraitComposition

    ^ true
! !

!RGTraitComposition class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

